import styled, { DefaultTheme, StyledComponent } from 'styled-components';

import defaultTheme from '../../theme';
import { paddable, flexbox, alignColumn, alignRow } from '../../utils';

export interface BoxProps {
  centered?: boolean;
  padded?: boolean;
  size?: number | number[];
  type?: 'row' | 'column';
  verticalAlign?: 'start' | 'center' | 'end';
  float?: 'left' | 'right';
  textAlign?: 'left' | 'right' | 'center' | 'justify';
}

export const Box = styled.div<BoxProps>(
  {
    display: 'flex',
    padding: '0.5em',
  },
  (props) => ({
    flexFlow: `${props.type} wrap`,
    ...(props.type === 'column' ? alignColumn(props) : alignRow(props)),
  }),
  ({ textAlign }) => (textAlign ? { textAlign } : null),
  flexbox,
  paddable,
);

Box.defaultProps = {
  theme: defaultTheme,
  type: 'column',
};

/**
 * Helpers for migrating from Semantic UI
 */

export type GridType = StyledComponent<
  'div',
  DefaultTheme,
  BoxProps & {
    type: string;
  },
  'type'
>;

export type GridParent = GridType & {
  Row: GridType;
  Column: GridType;
};

export const Grid: GridParent = styled(Box).attrs({ type: 'column' })(
  {},
) as any;

Grid.Row = styled(Box).attrs({ type: 'row' })({});
Grid.Column = styled(Box).attrs({ type: 'column' })({});
